<?php

/**
 * @file
 *
 * CTools content type to render a fielded panel pane.
 */
$plugin = array(
  'title' => t('Fielded custom content'),
  'no title override' => TRUE,
  'description' => t('Create custom panels pane with fields'),
  'category' => t('Fielded panes'),
  'all contexts' => TRUE,
  'defaults' => array(),
  'edit form' => 'fieldable_panels_panes_fieldable_panels_pane_content_type_edit_form',
  'edit text' => t('Edit'),
);

// --------------------------------------------------------------------------
// Callbacks, many of them automatically named, for rendering content.

/**
 * Callback to return the custom content types with the specified $subtype_name.
 */
function fieldable_panels_panes_fieldable_panels_pane_content_type_content_type($subtype_name) {
  $types = _fieldable_panels_panes_default_content_type();
  if (isset($types[$subtype_name])) {
    return $types[$subtype_name];
  }
  else {
    $entity = fieldable_panels_panes_load_entity($subtype_name);
    if ($entity) {
      return _fieldable_panels_panes_custom_content_type($entity);
    }
  }
}

/**
 * Callback to return all custom content types available.
 */
function fieldable_panels_panes_fieldable_panels_pane_content_type_content_types() {
  $types = _fieldable_panels_panes_default_content_type();

  $ids = db_query('SELECT fpid FROM {fieldable_panels_panes} WHERE reusable = 1')->fetchCol();
  if ($ids) {
    $entities = fieldable_panels_panes_load_multiple($ids);
    foreach ($entities as $entity) {
      $subtype = _fieldable_panels_panes_custom_content_type($entity);
      $types[$subtype['name']] = $subtype;
    }
  }

  return $types;
}

/**
 * Callback to render our content type.
 */
function fieldable_panels_panes_fieldable_panels_pane_content_type_render($subtype, $conf, $panel_args, $context) {
  $entity = fieldable_panels_panes_load_entity($subtype);

  if ($entity) {
    $block = new stdClass();
    $block->title = !empty($entity->title) ? filter_xss_admin($entity->title) : '';
    $block->content = fieldable_panels_pane_view($entity);

    return $block;
  }
}

/**
 * Callback to provide the administrative title of the custom content.
 */
function fieldable_panels_panes_fieldable_panels_pane_content_type_admin_title($subtype, $conf) {
  $entity = fieldable_panels_panes_load_entity($subtype);

  $output = t('Deleted/removed entity pane');
  if (isset($entity) && is_object($entity)) {
    $title = !empty($entity->admin_title) ? $entity->admin_title : (!empty($entity->title) ? $entity->title : t('No title'));

    $output = $title;
  }

  return $output;
}

/**
 * Form used to edit our content type.
 */
function fieldable_panels_panes_fieldable_panels_pane_content_type_edit_form($form, &$form_state) {
  $conf = &$form_state['conf'];
  $subtype_name = $form_state['subtype_name'];

  $entity = $form_state['entity'] = fieldable_panels_panes_load_entity($subtype_name);

  // It's possible that we have a reference to an entity that is no longer valid.
  // If so, bail, because otherwise field API will whitescreen.
  if (!$entity) {
    $form['error'] = array(
      '#markup' => t('The pane entity referenced does not appear to be valid. It was probably deleted and you should remove this pane.'),
    );
    return $form;
  }

  $form['#parameters'] = array('fieldable_panels_panes_fieldable_panels_pane_content_type_edit_form');

  $form = fieldable_panels_panes_entity_edit_form($form, $form_state);
  $form['reusable']['warning'] = array(
    '#markup' => '<div type="description">' . t('Note: Editing any value on a reusable pane will change the value everywhere this pane is used.') . '</div>',
  );

  return $form;
}

/**
 * Validate submission of our content type edit form.
 */
function fieldable_panels_panes_fieldable_panels_pane_content_type_edit_form_validate($form, &$form_state) {
  if ($form_state['entity']) {
    fieldable_panels_panes_entity_edit_form_validate($form, $form_state);
  }
}

/**
 * Submit our content type edit form.
 */
function fieldable_panels_panes_fieldable_panels_pane_content_type_edit_form_submit($form, &$form_state) {
  $entity = $form_state['entity'];
  if (!$entity) {
    return;
  }

  fieldable_panels_panes_entity_edit_form_submit($form, $form_state);
  if (!empty($entity->is_new)) {
    if (module_exists('uuid') && isset($entity->uuid)) {
      $entity_id = 'uuid:' . $entity->uuid;
    }
    else {
      $entity_id = 'fpid:' . $entity->fpid;
    }

    // @todo -- this won't work if $form_state does not contain
    // 'pane' which could theoretically happen in a non-Panels
    // use case. Not that anybody uses this outside of Panels.
    $form_state['pane']->subtype = $entity_id;
  }
}

// --------------------------------------------------------------------------
// Internal methods used by the above callbacks.

/**
 * Provide the default content types.
 *
 * These are all visible in the UI as the content types that allow a user
 * to add new panel pane entities that will then be stored in the database.
 */
function _fieldable_panels_panes_default_content_type() {
  $types = array();

  $entity_info = entity_get_info('fieldable_panels_pane');

  foreach ($entity_info['bundles'] as $bundle => $info) {
    $types[$bundle] = array(
      'name' => $bundle,
      'title' => $info['label'],
      'category' => t('Fielded panes'),
      'description' => t('Create a new custom entity.'),
      'all contexts' => TRUE,
      'bundle' => $bundle,
    );
  }

  return $types;
}

/**
 * Return an info array for a specific custom content type.
 */
function _fieldable_panels_panes_custom_content_type($entity) {
  $info = array(
    'title' => check_plain($entity->admin_title),
    'description' => check_plain($entity->admin_description),
    'category' => $entity->category ? check_plain($entity->category) : t('Miscellaneous'),
    'all contexts' => TRUE,
    'icon' => 'icon_block_custom.png',
  );

  if (module_exists('uuid') && isset($entity->uuid)) {
    $info['name'] = 'uuid:' . $entity->uuid;
  }
  else {
    $info['name'] = 'fpid:' . $entity->fpid;
  }

  $info['entity_id'] = $info['name'];
  return $info;
}

/**
 * Properly load the entity the subtype.
 */
function fieldable_panels_panes_load_entity($subtype_name) {
  $plugin = ctools_get_content_type('fieldable_panels_pane');

  // Next, check to see how the subtype is configured.
  $subtype_info = ctools_content_get_subtype($plugin, $subtype_name);

  // This means we're probably in the process of creating a new one.
  if (isset($subtype_info['bundle'])) {
    return fieldable_panels_pane_create(array('bundle' => $subtype_info['bundle']));
  }

  // And try it this way.
  if (isset($subtype_info['entity_id'])) {
    return _fieldable_panels_panes_load_entity($subtype_info['entity_id']);
  }

  // Finally, try this:
  return _fieldable_panels_panes_load_entity($subtype_name);
}

/**
 * Properly load the entity via $subtype_name
 */
function _fieldable_panels_panes_load_entity($subtype_name) {
  list($type, $id) = explode(':', $subtype_name);
  if ($type == 'uuid' && module_exists('uuid')) {
    $content = entity_uuid_load($id);
  }
  else {
    $content = fieldable_panels_panes_load($id);
  }

  return $content;
}
